window.Gleam.initSearch([{"doc":"outcome","title":"outcome","content":"# outcome\n\n[![Package Version](https://img.shields.io/hexpm/v/outcome)](https://hex.pm/packages/outcome)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/outcome/)\n\nAn Error Handling library for Gleam. Inspired by\n&lt;https://github.com/lpil/snag&gt; and &lt;https://effect.website/&gt;.\n\nOutcomes gives you:\n\n- An error stack that makes it easier to track the path of an error.\n- A way to distinguish between unexpected errors (Defect) and expected errors (Failure).\n\n```sh\ngleam add outcome\n```\n\n```gleam\nimport outcome.{type Outcome}\n\nfn using(email) {\n  case signup(email) {\n    Error(stack) -&gt; io.print(outcome.pretty_print(stack))\n    Ok(user) -&gt; {\n      //...\n    }\n  }\n}\n\nfn signup(email: String) -&gt; Outcome(User) {\n  use valid_email &lt;- result.try(\n    validate_email(email)\n    |&gt; outcome.with_context(&quot;In signup&quot;)\n  )\n\n  create_user(valid_email)\n}\n\n// An expected error should be marked as a failure\nfn validate_email(email: String) -&gt; Outcome(String) {\n  Error(&quot;Invalid email&quot;)\n    |&gt; outcome.into_failure\n}\n\n// An unexpected error should be marked as a defect\nfn create_user() -&gt; Outcome(User) {\n  Error(&quot;Some SQL error&quot;)\n  |&gt; outcome.into_defect\n  |&gt; outcome.with_context(&quot;In create_user&quot;)\n}\n```\n\n```gleam\nusing(&quot;invalid&quot;)\n\nFailure: Invalid email\n\nstack:\n  Context: In signup\n  Failure: Invalid email\n```\n\n```gleam\nusing(&quot;sam@sample.com&quot;)\n\nDefect: Some SQL error\n\nstack:\n  Context: In signup\n  Context: In create_user\n  Defect: Some SQL error\n```\n\n## Notes\n\n- When you push a defect on top of a failure, then the `Problem` becomes a defect.\n- When a problem is a `Defect` it stays a `Defect` unless explicitly changed.\n\nFurther documentation can be found at &lt;https://hexdocs.pm/outcome&gt;.\n","url":"index.html"},{"doc":"outcome","title":"Outcome","content":"pub type Outcome(t) =\n  Result(t, Problem)\n Alias to Result with Problem as error type.\n\nSynonyms:\noutcome.Outcome\noutcome Outcome","url":"outcome.html#Outcome"},{"doc":"outcome","title":"Problem","content":"pub type Problem {\n  Defect(message: String, stack: Stack)\n  Failure(message: String, stack: Stack)\n}\n The error type ie. `Result(t, Problem)`\n An application error is either a Defect or a Failure.\n A Defect is an unexpected application error, which shouldn&#39;t be shown to the user.\n A Failure is an expected error.\nDefect(message: String, stack: Stack)\n\n\nFailure(message: String, stack: Stack)\n\n\nSynonyms:\noutcome.Problem\noutcome Problem","url":"outcome.html#Problem"},{"doc":"outcome","title":"Stack","content":"pub type Stack =\n  NonEmptyList(StackEntry)\n\n\nSynonyms:\noutcome.Stack\noutcome Stack","url":"outcome.html#Stack"},{"doc":"outcome","title":"StackEntry","content":"pub type StackEntry {\n  StackEntryContext(String)\n  StackEntryDefect(String)\n  StackEntryFailure(String)\n}\n Stax entries\n Context is just information about the place in the application to build a stack trace.\nStackEntryContext(String)\n\n\nStackEntryDefect(String)\n\n\nStackEntryFailure(String)\n\n\nSynonyms:\noutcome.StackEntry\noutcome StackEntry","url":"outcome.html#StackEntry"},{"doc":"outcome","title":"add_context_to_problem","content":"pub fn add_context_to_problem(\n  problem: Problem,\n  value: String,\n) -&gt; Problem\n A context to a Problem\nSynonyms:\noutcome.add_context_to_problem\noutcome add_context_to_problem","url":"outcome.html#add_context_to_problem"},{"doc":"outcome","title":"add_defect_to_problem_stack","content":"pub fn add_defect_to_problem_stack(\n  problem: Problem,\n  failure: String,\n) -&gt; Problem\n\nSynonyms:\noutcome.add_defect_to_problem_stack\noutcome add_defect_to_problem_stack","url":"outcome.html#add_defect_to_problem_stack"},{"doc":"outcome","title":"add_failure_to_problem_stack","content":"pub fn add_failure_to_problem_stack(\n  problem: Problem,\n  failure: String,\n) -&gt; Problem\n\nSynonyms:\noutcome.add_failure_to_problem_stack\noutcome add_failure_to_problem_stack","url":"outcome.html#add_failure_to_problem_stack"},{"doc":"outcome","title":"add_to_problem_stack","content":"pub fn add_to_problem_stack(\n  problem: Problem,\n  stack_entry: StackEntry,\n) -&gt; Problem\n Add an StackEntry to the top of a Problem stack\nSynonyms:\noutcome.add_to_problem_stack\noutcome add_to_problem_stack","url":"outcome.html#add_to_problem_stack"},{"doc":"outcome","title":"as_defect","content":"pub fn as_defect(\n  result: Result(a, Nil),\n  e: String,\n) -&gt; Result(a, Problem)\n Replaces an `Error(Nil)` with an `Error(Defect)`\nSynonyms:\noutcome.as_defect\noutcome as_defect","url":"outcome.html#as_defect"},{"doc":"outcome","title":"as_failure","content":"pub fn as_failure(\n  result: Result(a, Nil),\n  e: String,\n) -&gt; Result(a, Problem)\n Replaces an Error(Nil) with an Error(Failure)\nSynonyms:\noutcome.as_failure\noutcome as_failure","url":"outcome.html#as_failure"},{"doc":"outcome","title":"error_with_defect","content":"pub fn error_with_defect(defect: String) -&gt; Result(a, Problem)\n Create a Defect wrapped in an Error\nSynonyms:\noutcome.error_with_defect\noutcome error_with_defect","url":"outcome.html#error_with_defect"},{"doc":"outcome","title":"error_with_failure","content":"pub fn error_with_failure(failure: String) -&gt; Result(a, Problem)\n Create Failure wrapped in an Error\nSynonyms:\noutcome.error_with_failure\noutcome error_with_failure","url":"outcome.html#error_with_failure"},{"doc":"outcome","title":"into_defect","content":"pub fn into_defect(\n  result: Result(a, String),\n) -&gt; Result(a, Problem)\n Convert an `Error(String)` into an `Error(Defect)`\nSynonyms:\noutcome.into_defect\noutcome into_defect","url":"outcome.html#into_defect"},{"doc":"outcome","title":"into_failure","content":"pub fn into_failure(\n  result: Result(a, String),\n) -&gt; Result(a, Problem)\n Convert an `Error(String)` into an `Error(Failure)`\nSynonyms:\noutcome.into_failure\noutcome into_failure","url":"outcome.html#into_failure"},{"doc":"outcome","title":"map_into_defect","content":"pub fn map_into_defect(\n  result: Result(a, b),\n  mapper: fn(b) -&gt; String,\n) -&gt; Result(a, Problem)\n Convert an `Error(t)` into a wrapped Defect, by using a mapping function\nSynonyms:\noutcome.map_into_defect\noutcome map_into_defect","url":"outcome.html#map_into_defect"},{"doc":"outcome","title":"new_defect","content":"pub fn new_defect(message: String) -&gt; Problem\n\nSynonyms:\noutcome.new_defect\noutcome new_defect","url":"outcome.html#new_defect"},{"doc":"outcome","title":"new_failure","content":"pub fn new_failure(message: String) -&gt; Problem\n\nSynonyms:\noutcome.new_failure\noutcome new_failure","url":"outcome.html#new_failure"},{"doc":"outcome","title":"pretty_print","content":"pub fn pretty_print(problem: Problem) -&gt; String\n\nSynonyms:\noutcome.pretty_print\noutcome pretty_print","url":"outcome.html#pretty_print"},{"doc":"outcome","title":"problem_with_defect","content":"pub fn problem_with_defect(\n  problem: Problem,\n  defect_message: String,\n) -&gt; Problem\n\nSynonyms:\noutcome.problem_with_defect\noutcome problem_with_defect","url":"outcome.html#problem_with_defect"},{"doc":"outcome","title":"problem_with_failure","content":"pub fn problem_with_failure(\n  problem: Problem,\n  failure_message: String,\n) -&gt; Problem\n\nSynonyms:\noutcome.problem_with_failure\noutcome problem_with_failure","url":"outcome.html#problem_with_failure"},{"doc":"outcome","title":"with_context","content":"pub fn with_context(\n  outcome outcome: Result(a, Problem),\n  context context: String,\n) -&gt; Result(a, Problem)\n Add context to an Outcome\nSynonyms:\noutcome.with_context\noutcome with_context","url":"outcome.html#with_context"},{"doc":"outcome","title":"with_defect","content":"pub fn with_defect(\n  outcome outcome: Result(a, Problem),\n  defect_message defect_message: String,\n) -&gt; Result(a, Problem)\n\nSynonyms:\noutcome.with_defect\noutcome with_defect","url":"outcome.html#with_defect"},{"doc":"outcome","title":"with_failure","content":"pub fn with_failure(\n  outcome outcome: Result(a, Problem),\n  failure_message failure_message: String,\n) -&gt; Result(a, Problem)\n\nSynonyms:\noutcome.with_failure\noutcome with_failure","url":"outcome.html#with_failure"},{"doc":"outcome","title":"outcome","content":"","url":"outcome.html"}]);